<?php

//
// innovaphone mailbox PHP class
// copyright (c) innovaphone 2012 - 2014
//

include('Mail.php');
include('Mail/mime.php');

require_once('innovaphone.mail.info.class.php');
require_once('innovaphone.mail.class.php');

class InnovaphoneMailbox {

    protected $host;
    protected $port;
    protected $username;
    protected $password;
    protected $folder;
    protected $connection;
    protected $error;

    // class constructor
    public function __construct($host, $port, $username, $password, $folder = "INBOX", $connect = true) {
        $this->host       = $host;
        $this->port       = $port;
        $this->username   = $username;
        $this->password   = $password;
        $this->folder     = $folder;
        $this->connection = null;
        $this->error      = "";
        if($connect) {
            $this->connect();
        }
    }

    public function connect() {
        //$extra = ":995/pop3/ssl/novalidate-cert";
        //$extra = ":110/pop3/notls";
        //$extra = ":143/imap/notls";
        if($this->connection == null) {
            $extra = "/pop3/ssl/novalidate-cert";
            $mailbox = "{" . "{$this->host}:{$this->port}" . $extra . "}" . "{$this->folder}";
            $this->connection = imap_open($mailbox, $this->username, $this->password);
            if($this->connection === false) {
                //var_dump(imap_errors());
            }
        }
        return $this->connection !== false;
    }

    public function disconnect() {
        $ret = false;
        if($this->connection) {
            $ret = imap_close($this->connection);
            $this->connection = null;
        }
        return $ret;
    }

    public function isConnected() {
        return $this->connection != false;
    }
    
    public function getStatus() {
        $status = imap_status($this->connection, $this->folder, SA_ALL);
        $ret = "";
        if($status) {
            $ret .= "Messages:   " . $status->messages    . "\n";
            $ret .= "Recent:     " . $status->recent      . "\n";
            $ret .= "Unseen:     " . $status->unseen      . "\n";
            $ret .= "UIDnext:    " . $status->uidnext     . "\n";
            $ret .= "UIDvalidity:" . $status->uidvalidity . "\n";
        } else {
            $ret .= "imap_status failed: " . imap_last_error() . "\n";
        }
        return $ret;
    }

    public function getErrors() {
        $ret = "";
        foreach(imap_errors() as $line) {
            $ret .= $line . "\n";
        }
        if($this->error != "") {
            $ret .= $this->error ."\n";
            $this->error = "";
        }
        return $ret;
    }

    public function getMailInfos($unseen = false, $max = 100) {
        $ret = array();
        if($this->isConnected()) {        
            $mailboxInfo = imap_check($this->connection);
            if($mailboxInfo === false) {
                return false;
            }
            if($mailboxInfo->Nmsgs) {
                $range = "1:" . min($max, $mailboxInfo->Nmsgs);
                $overview = imap_fetch_overview($this->connection, $range);
                foreach($overview as $mailHeader) {
                    if(array_key_exists("message_id", get_object_vars($mailHeader)) &&
                       array_key_exists("from", get_object_vars($mailHeader)) &&
                       count($mailHeader->from) &&
                       (!$unseen || !$mailHeader->seen))
                    {
                        $senders = imap_rfc822_parse_adrlist($mailHeader->from, "");
                        $ret[] = new InnovaphoneMailInfo($mailHeader->message_id, $senders[0], $mailHeader->recent, $mailHeader->seen, $mailHeader->answered);
                    }
                }
            }
            return $ret;
        }
        return false;
    }

    public function getMail($messageId) {
        $messageNumber = $this->findMessageNumber($messageId);
        if($messageNumber) {
            $mailHeader = imap_fetchheader($this->connection, $messageNumber);
            $mailHeaderRFC822   = imap_rfc822_parse_headers($mailHeader);
            if(array_key_exists("message_id", get_object_vars($mailHeaderRFC822)) &&
               $mailHeaderRFC822->message_id == $messageId &&
               array_key_exists("from", get_object_vars($mailHeaderRFC822)) &&
               count($mailHeaderRFC822->from))
            {
                $mailStructure          = imap_fetchstructure($this->connection, $messageNumber);
                $mailTextPlain          = null;
                $mailTextHtml           = null;
                $mailTextHtmlCharset    = "";
                $attachments            = $this->parseSections($mailStructure, "");
                $to                     = array_key_exists("to", get_object_vars($mailHeaderRFC822)) ? $mailHeaderRFC822->to : "";
                $cc                     = array_key_exists("cc", get_object_vars($mailHeaderRFC822)) ? $mailHeaderRFC822->cc : array();
                $subject                = array_key_exists("subject", get_object_vars($mailHeaderRFC822)) ? iconv_mime_decode($mailHeaderRFC822->subject, 0, "UTF-8") : "";
                
                //var_dump($mailHeader);
                //var_dump($mailStructure);
                
                foreach($to as &$recipient) {
                    $recipient->personal = isset($recipient->personal) ? imap_utf8($recipient->personal) : "";
                }
                if(is_array($cc)) {
                    foreach($cc as &$recipient) {
                        $recipient->personal = isset($recipient->personal) ? imap_utf8($recipient->personal) : "";
                    }
                }

                foreach($attachments as $key => &$attachment) {
                    //print "Section: " . $attachment->getId() . ", Type: " . $attachment->getType() . ", Subtype: " . $attachment->getSubtype() . "\n";
                    if($attachment->getType() == 0 && $attachment->getFileName() == "") {
                        $data = imap_fetchbody($this->connection, $messageNumber, $attachment->getId());
                        switch($attachment->getEncoding()) {
                            case 3: 
                                $data = imap_base64($data);
                                break;
                            case 4:
                                $data = quoted_printable_decode($data);
                                break;
                        }
                        if($attachment->getSubtype() == 'plain') {
                            if(function_exists('iconv')) {
                                $mailTextPlainCharset = $attachment->getCharset();
                                $partCharset = $this->getCharset($messageId, $attachment->getId());
                                if($partCharset != "") {
                                    /*if(strcasecmp($partCharset, $mailTextPlainCharset)) {
                                        print("Text Mime charset difference: \"$mailTextPlainCharset\", \"$partCharset\", using \"$partCharset\"\n");
                                    }
                                    else {
                                        print("Text charset: \"$partCharset\"\n");
                                    }*/
                                    $mailTextPlainCharset = $partCharset;
                                }
                                if($mailTextPlainCharset != "") {
                                    $mailTextPlain = iconv($mailTextPlainCharset, 'utf-8', $data);
                                    if($mailTextPlain === false) {
                                        $mailTextPlain = imap_utf8($data);
                                    }
                                }
                                else {
                                    $mailTextPlain = imap_utf8($data);
                                }
                            }
                            else {
                                $mailTextPlain = imap_utf8($data);
                            }
                        }
                        else if($attachment->getSubtype() == 'html') {
                            $mailTextHtml = $data;
                            $mailTextHtmlCharset = $attachment->getCharset();

                            $partCharset = $this->getCharset($messageId, $attachment->getId());
                            if($partCharset != "") {                            
                                /*if(strcasecmp($partCharset, $mailTextHtmlCharset)) {
                                    print("HTML Mime charset difference: \"$mailTextHtmlCharset\", \"$partCharset\", using \"$partCharset\"\n");
                                }
                                else {
                                    print("HTML charset: \"$partCharset\"\n");
                                }*/
                                $mailTextHtmlCharset = $partCharset;
                            }
                        }
                        unset($attachments[$key]);
                    }
                }
                $mail = new InnovaphoneMail($mailHeaderRFC822->message_id,
                    $mailHeaderRFC822->from[0],
                    $to,
                    $cc,
                    $subject,
                    $mailTextPlain,
                    $mailTextHtml,
                    $mailTextHtmlCharset,
                    $attachments);
                
                $mail->setDate(new DateTime($mailHeaderRFC822->date));

                $matches = null;
                preg_match_all('/([^: ]+): (.+?(?:\r\n\s(?:.+?))*)\r\n/m', $mailHeader, $matches);
                
                foreach($matches[1] as $count => $header) {
                    $value = $matches[2][$count];
                    
                    if($header == "Importance") {
                        $mail->setImportance($value);
                    }
                    if($header == "Priority") {
                        $mail->setPriority($value);
                    }
                    if($header == "X-Priority") {
                        $mail->setXPriority($value);
                    }
                }

                return $mail;
            }
        }
        return false;
    }

    public function getCharset($messageId, $partId) {
        $messageNumber = $this->findMessageNumber($messageId);
        if($messageNumber) {
            $mime_data = imap_fetchmime($this->connection, $messageNumber, $partId);
            $mime_data_rows = explode("\r", str_replace("\n", "\r", $mime_data));
            foreach($mime_data_rows as $mime_row) {
                $mime_data_columns = explode(" ", trim($mime_row));
                foreach($mime_data_columns as $mime_column) {
                    $charset = stristr($mime_column, "charset=");
                    if($charset !== false) {
                        $charset = trim(substr($charset, 8), "\" \r\n");
                        return $charset;
                    }
                }
            }
        }
        return false;
    }
    
    public function setFlag($messageId, $seen, $answered = false) {
        $messageNumber = $this->findMessageNumber($messageId);
        if($messageNumber && ($seen || $answered)) {
            return imap_setflag_full($this->connection, "$messageNumber", ($seen ? "\\Seen " : "") . ($answered ? "\\Answered" : ""));
        }
        return false;
    }

    public function clearFlag($messageId, $seen, $answered = false) {
        $messageNumber = $this->findMessageNumber($messageId);
        if($messageNumber && ($seen || $answered)) {
            return imap_clearflag_full($this->connection, "$messageNumber", ($seen ? "\\Seen " : "") . ($answered ? "\\Answered" : ""));
        }
        return false;
    }

    public function getHeader($messageId) {
        $messageNumber = $this->findMessageNumber($messageId);
        if($messageNumber) {
            return imap_fetchheader($this->connection, $messageNumber);
        }
        return false;
    }
    
    public function getBody($messageId) {
        $messageNumber = $this->findMessageNumber($messageId);
        if($messageNumber) {
            return imap_body($this->connection, $messageNumber);
        }
        return false;
    }
    
    public function deleteMail($messageId) {
        $messageNumber = $this->findMessageNumber($messageId);
        if($messageNumber) {
            imap_delete($this->connection, $messageNumber, CL_EXPUNGE);
            imap_expunge($this->connection);
            return true;
        }
        return false;
    }

    public function saveAttachment(&$mail, &$attachment, $fileName = "") {
        $messageNumber = $this->findMessageNumber($mail->getMessageId());
        if($messageNumber) {
            if($fileName == "") {
                $fileName = $attachment->getFileName();    
            }
            $fileHandle = fopen($fileName, "wb");
            if(!$fileHandle) {
                return false;   
            }
            $data = imap_fetchbody($this->connection, $messageNumber, $attachment->getId());
            switch($attachment->getEncoding()) {
                case 3: 
                    $data = imap_base64($data);
                    break;
                case 4:
                    $data = quoted_printable_decode($data);
                    break;
            }
            if(!fwrite($fileHandle, $data)) {
                return false;
            }
            fclose($fileHandle);            
            return true;
        }
        return false;
    }
    
    static public function saveMail($outputFileName, $to, $from, $subject, $message, $files = array(), $replyToMessageId = null) {
        $headers                = array();
        $headers['From']        = $from;
        $headers['To']          = $to;
        $headers['Subject']     = $subject;
        if($replyToMessageId) {
            $headers['In-Reply-To'] = $replyToMessageId;
        }
        $headers['Precedence']                  = "bulk";
        $headers['Auto-Submitted']              = "auto-replied";
        $headers['X-Auto-Response-Suppress']    = "DR,RN,NRN,OOF,AutoReply";
        
        $mime                   = new Mail_Mime(array('eol' => "\n"));
        
        $mime->setTXTBody($message);

        foreach($files as $fileName => $file) {
            $mime->addAttachment($file, 'application/octet-stream', $fileName, true, "base64", "attachment", "", "", "", "", "", "", "UTF-8");
        }

        $mimeparams = array();
        $mimeparams['text_encoding']    = "8bit";
        $mimeparams['text_charset']     = "UTF-8";
        $mimeparams['html_charset']     = "UTF-8";
        $mimeparams['head_charset']     = "UTF-8";

        $body                   = $mime->get($mimeparams, $outputFileName);
        $headers                = $mime->headers($headers);
        
        return $headers;
    }
    
    public function sendMail($to, $from, $subject, $message, $files = array(), $replyToMessageId = null) {
        $params['host']         = $this->host;
        $params['port']         = "25";
        $params['auth']         = false;
        $params['username']     = $this->username;
        $params['password']     = $this->password;

        $headers                = array();
        $headers['From']        = $from;
        $headers['To']          = $to;
        $headers['Subject']     = $subject;
        if($replyToMessageId) {
            $headers['In-Reply-To'] = $replyToMessageId;
        }
        $headers['Precedence']                  = "bulk";
        $headers['Auto-Submitted']              = "auto-replied";
        $headers['X-Auto-Response-Suppress']    = "DR,RN,NRN,OOF,AutoReply";
            
        $mime                   = new Mail_Mime(array('eol' => "\n"));
        
        $mime->setTXTBody($message);

        foreach($files as $fileName => $file) {
            $mime->addAttachment($file, 'application/octet-stream', $fileName, true, "base64", "attachment", "", "", "", "", "", "", "UTF-8");
        }

        $mimeparams = array();
        $mimeparams['text_encoding']    = "8bit";
        $mimeparams['text_charset']     = "UTF-8";
        $mimeparams['html_charset']     = "UTF-8";
        $mimeparams['head_charset']     = "UTF-8";

        $body                   = $mime->get($mimeparams);
        $headers                = $mime->headers($headers);

        try {
            $mail_object        = &Mail::factory("smtp", $params);
            $ret                = $mail_object->send($to, $headers, $body);
            if($ret !== true) {
                $this->error = $ret->getMessage()." (".$ret->getCode().")";
                $ret = false;
            }
        }
        catch(Exception $e) {
            $ret = false;
            $this->error = $e->getMessage()." (".$e->getCode().")";
        }
        
        return $ret;
    }
        
    public function responseMail(&$mail, $message) {
        $to                     = imap_rfc822_write_address($mail->getSender()->mailbox,
                                                            $mail->getSender()->host,
                                                            isset($mail->getSender()->personal) ? $mail->getSender()->personal : "");
        $subject                = "Re: " . $mail->getSubject();
        $recipients             = $mail->getRecipients();
        if(count($recipients)) {
            $from               = imap_rfc822_write_address($recipients[0]->mailbox, $recipients[0]->host, $recipients[0]->personal);
        }
        else {
            $from               = imap_rfc822_write_address("faxserver", $mail->getSender()->host, "");
        }

        return $this->sendMail($to, $from, $subject, $message, array(), $mail->getMessageId());
    }
    
    private function parseSections($part, $id) {
        $attachments = array();
        if(($part->type == 0 && strtolower($part->subtype) == 'plain') ||
           ($part->type == 0 && strtolower($part->subtype) == 'html') ||
           $part->type == 3 ||
           (isset($part->disposition) && strtolower($part->disposition) == 'attachment'))
        {
            $parameters = array();
            if($part->ifdparameters) {
                foreach($part->dparameters as $dparameter) {
                    $parameters[strtolower($dparameter->attribute)] = utf8_decode(imap_utf8($dparameter->value));
                }
            }
            if($part->ifparameters) {
                foreach($part->parameters as $parameter) {
                    $parameters[strtolower($parameter->attribute)] = utf8_decode(imap_utf8($parameter->value));
                }
            }
            $attachments[] = new InnovaphoneMailAttachment($id == "" ? "1" : $id, strtolower($part->type), strtolower($part->subtype), $part->encoding, $parameters);
        }
        if(isset($part->parts)) {
            foreach($part->parts as $count => $partOfPart) {
                if($partOfPart->type != 2) {
                    $attachments = array_merge($attachments, $this->parseSections($partOfPart, $id . ($id == "" ? "" : ".") . ($count + 1)));
                }
            }
        }
        return $attachments;
    }

    private function findMessageNumber($messageId) {
        $max = 100;
        if($this->isConnected()) {        
            $mailboxInfo = imap_check($this->connection);
            $counter = 0;
            while($counter < $mailboxInfo->Nmsgs) {
                $range = ($counter + 1) . ":" . (min($max, $mailboxInfo->Nmsgs - $counter) + $counter);
                $overview = imap_fetch_overview($this->connection, $range);
                foreach($overview as $mailHeader) {
                    if(array_key_exists("message_id", get_object_vars($mailHeader)) 
                        && $mailHeader->message_id == $messageId)
                    {
                        return $mailHeader->msgno;
                    }
                }
                $counter += $max;
            }
            return false;
        }
        return false;
    }
}

?>